#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "config.h"
#include "data.h"
#include "xmlparser.h"

Config::Config()
{
  strcpy(IpTables, "/sbin/iptables");
  strcpy(RrdTool, "/usr/bin/rrdtool");
  strcpy(DataDir, "/var/www/html/ipaccounting/");
  strcpy(DataFileName, "data.xml");
  strcpy(OldDataFileName, "data.xml");
  DataFormat = DATA_XML;
  OldDataFormat = DATA_TEXT;
  strcpy(HTMLTemplate, "/var/www/html/ipaccounting/index.htmx");
  WWWRefresh = 300;
  strcpy(GraphScriptURL, "/ipaccounting/rrd.php");
  TestRun = false;
  Debug = true;
  DeviceNum = 0;
  InputNum = 0;
  RemoveUnlistedAddresses = false;
  DeleteRRDs = false;
}

void Config::Print()
{
  unsigned int i;

  printf("IpTables: %s\n", IpTables);
  printf("RrdTool: %s\n", RrdTool);
  printf("DataDir: %s\n", DataDir);

  printf("Inputs: %u\n", InputNum);
  for(i = 0; i < InputNum; i++)
    printf(" type:%u name:%s\n", Input[i].type, Input[i].name);
    
  printf("DataFileName: %s\n", DataFileName);
  printf("DataFormat: %d\n", DataFormat);
  printf("OldDataFormat: %d\n", OldDataFormat);
  printf("HTMLTemplate: %s\n", HTMLTemplate);
  printf("WWWRefresh: %d s\n", WWWRefresh);
  printf("GraphScriptURL: %s\n", GraphScriptURL);

  printf("Devices: %u", DeviceNum);
  for (i = 0; i < DeviceNum; i++)
    printf(" %s", DeviceList[i]);
  printf("\n");

  printf("RemoveUnlistedAddresses: %d\n", RemoveUnlistedAddresses);
  printf("DeleteRRDs: %d\n", DeleteRRDs);

  printf("\n");
}

bool Config::setIpTablesLocation(const char *location)
{
  strcpy(IpTables, location);
  return true;
}

bool Config::setRrdToolLocation(const char *location)
{
  strcpy(RrdTool, location);
  return true;
}

bool Config::setDataDirLocation(const char *location)
{
  strcpy(DataDir, location);
  return true;
}

bool Config::addInputType(const char *type)
{
  if(InputNum == (MAX_INPUTS - 1))
  {
    fprintf(stderr,"Too much inputs, limit is %d.\n", MAX_INPUTS);
    exit(-10);
  }

  unsigned ntype;
  if(strcasecmp(type, "arp") == 0)
    ntype = INPUT_ARP;
  else if(strcasecmp(type, "lbcfree") == 0)
    ntype = INPUT_LBCFREE;
  else if(strcasecmp(type, "list") == 0)
    ntype = INPUT_LIST;
  else
  {
    fprintf(stderr,"Unknown input type: %s\n", type);
    exit(-11);
  }
  
  Input[InputNum].type = ntype;
  InputNum++;
  
  return true;
}

bool Config::addInputName(const char *name)
{
  strcpy(Input[InputNum-1].name, name);
  return true;
}

bool Config::addDevice(const char *device)
{
  char *devname = strtok((char *) device, " ,");

  while (devname && *devname)
  {
    if (DeviceNum + 1 == DEVICES)
    {
      fprintf(stderr, "Device table full.\n");
      return false;
    }
    if (strlen(devname) >= DEVNAME_SIZE)
    {
      fprintf(stderr, "Device name %s too long.\n", devname);
      return false;
    }

    strcpy(DeviceList[DeviceNum], devname);
    DeviceNum++;
    devname = strtok(NULL, " ,");
  }

  return true;
}

bool Config::setDataFileName(const char *name)
{
  strcpy(DataFileName, name);
  return true;
}

bool Config::setOldDataFileName(const char *name)
{
  strcpy(OldDataFileName, name);
  return true;
}

int Config::parseDataFormat(const char *format)
{
  if (strcasecmp(format, "xml") == 0)
    return DATA_XML;
  else if (strcasecmp(format, "text") == 0)
    return DATA_TEXT;
  else if (strcasecmp(format, "binary") == 0)
    return DATA_BINARY;
  else if (strcasecmp(format, "rrd") == 0)
    return DATA_RRD;
  else
    return -1;
}

bool Config::setDataFormat(const char *format)
{
  int f = parseDataFormat(format);
  if (f < 0)
    return false;

  DataFormat = f;
  
  return true;
}

bool Config::setOldDataFormat(const char *format)
{
  int f = parseDataFormat(format);
  if (f < 0)
    return false;

  OldDataFormat = f;
  return true;
}

bool Config::setHTMLTemplate(const char *name)
{
  strcpy(HTMLTemplate, name);
  return true;
}

bool Config::setWWWRefresh(const char *refresh)
{
  unsigned ref = atoi(refresh);

  if (ref < 0)
    return false;

  WWWRefresh = ref;
  return true;
}

bool Config::setGraphScriptURL(const char *url)
{
  strcpy(GraphScriptURL, url);
  return true;
}

bool Config::setRemoveUnlistedAddresses(const char *value)
{
  if(strcasecmp(value,"yes")==0)
    RemoveUnlistedAddresses = true;

  return true;
}

bool Config::setDeleteRRDs(const char*value)
{
  if(strcasecmp(value,"yes")==0)
     DeleteRRDs = true;

  return true;
}

int Config::Read(char* file)
{
  XMLParser *xmlp = new XMLParser();
  xmlp->parseConfigFile(this, file);
  delete xmlp;

  return 1;
}

